/**
 * @semantic-ui/query v0.11.2
 * DOM querying and manipulation library for Semantic UI
 *
 * @license ISC
 * @copyright (c) 2025 Jack Lukic <jack@semantic-ui.com>
 * @homepage https://next.semantic-ui.com
 *
 * This source code is licensed under the ISC license found in the
 * LICENSE file in the root directory of this source tree.
 */
var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);

// ../utils/src/types.js
var isObject = (x) => {
  return x !== null && typeof x == "object";
};
var isPlainObject = (x) => {
  return isObject(x) && x.constructor === Object;
};
var isString = (x) => {
  return typeof x == "string";
};
var isArray = (x) => {
  return Array.isArray(x);
};
var isFunction = (x) => {
  return typeof x == "function" || false;
};
var isDOM = (element) => {
  if (typeof window === "undefined") {
    return true;
  }
  return element instanceof Element || element instanceof Document || element === window || element instanceof DocumentFragment;
};

// ../utils/src/looping.js
var each = (obj, func, context) => {
  if (obj === null || obj === void 0) {
    return obj;
  }
  const iteratee = context ? func.bind(context) : func;
  if (isObject(obj) || isFunction(obj)) {
    if (obj.length !== void 0 && typeof obj.length === "number") {
      obj = Array.from(obj);
    }
  }
  if (isArray(obj)) {
    for (let i = 0; i < obj.length; ++i) {
      if (iteratee(obj[i], i, obj) === false) {
        break;
      }
    }
  } else {
    const keys = Object.keys(obj);
    for (let key of keys) {
      if (iteratee(obj[key], key, obj) === false) {
        break;
      }
    }
  }
  return obj;
};

// ../utils/src/arrays.js
var findIndex = (array = [], callback) => {
  let matchedIndex = -1;
  each(array, (value, index) => {
    if (callback(value, index, array)) {
      matchedIndex = index;
      return false;
    }
  });
  return matchedIndex;
};
var inArray = (value, array = []) => {
  return array.indexOf(value) > -1;
};

// ../utils/src/ssr.js
var isClient = /* @__PURE__ */ (() => {
  return typeof window !== "undefined";
})();

// ../utils/src/colors.js
var GAMMA_EXPONENT = 1 / 2.4;

// ../utils/src/strings.js
var camelToKebab = (str = "") => {
  return str.replace(/([a-z0-9])([A-Z])/g, "$1-$2").toLowerCase();
};

// src/query.js
var _Query = class _Query {
  constructor(selector, { root = document, pierceShadow = false, prevObject = null } = {}) {
    let elements = [];
    if (!root) {
      return;
    }
    if (selector instanceof _Query) {
      elements = selector;
    }
    if (selector === window || selector === globalThis || inArray(selector, ["window", "globalThis"]) || selector == _Query.globalThisProxy) {
      elements = [_Query.globalThisProxy];
      this.isBrowser = isClient;
      this.isGlobal = true;
    } else if (isArray(selector) || selector instanceof NodeList || selector instanceof HTMLCollection) {
      selector = Array.from(selector);
      elements = selector;
    } else if (isString(selector)) {
      if (selector.trim().slice(0, 1) == "<") {
        const template = document.createElement("template");
        template.innerHTML = selector.trim();
        elements = Array.from(template.content.childNodes);
      } else {
        elements = pierceShadow ? this.querySelectorAllDeep(root, selector) : root.querySelectorAll(selector);
      }
    } else if (isDOM(selector)) {
      elements = [selector];
    } else if (selector instanceof NodeList) {
      elements = selector;
    }
    this.selector = selector;
    this.length = elements.length;
    this.options = { root, pierceShadow };
    this.prevObject = prevObject;
    Object.assign(this, elements);
  }
  /* Returns a copy of current DOM Object */
  chain(elements) {
    return this.isGlobal && !elements ? new _Query(globalThis, this.options) : new _Query(elements, { ...this.options, prevObject: this });
  }
  end() {
    return this.prevObject || this;
  }
  /* we will add all elements across shadow root boundaries while matching
     intermediate selectors at shadow boundaries
  */
  querySelectorAllDeep(root, selector, includeRoot = true) {
    const elements = /* @__PURE__ */ new Set();
    const domSelector = isDOM(selector);
    let domFound = false;
    let queriedRoot = false;
    if (includeRoot) {
      if (domSelector && root == selector || !domSelector && root.matches && root.matches(selector)) {
        elements.add(root);
      }
    }
    if (domSelector) {
      queriedRoot = true;
    } else if (root.querySelectorAll) {
      root.querySelectorAll(selector).forEach((el) => elements.add(el));
      queriedRoot = true;
    }
    const getRemainingSelector = (el, selector2) => {
      const parts = selector2.split(" ");
      let partialSelector;
      let remainingSelector;
      each(parts, (part, index) => {
        partialSelector = parts.slice(0, index + 1).join(" ");
        if (el.matches(partialSelector)) {
          remainingSelector = parts.slice(index + 1).join(" ");
          return;
        }
      });
      return remainingSelector || selector2;
    };
    const addElements = (node, selector2) => {
      if (domSelector && (node === selector2 || node.contains)) {
        if (node.contains(selector2)) {
          elements.add(selector2);
          domFound = true;
        }
      } else if (node.querySelectorAll) {
        node.querySelectorAll(selector2).forEach((el) => elements.add(el));
      }
    };
    const findElements = (node, selector2, query) => {
      if (domSelector && domFound) return;
      if (query === true) {
        addElements(node, selector2);
        queriedRoot = true;
      }
      if (node.nodeType === Node.ELEMENT_NODE && node.shadowRoot) {
        const newSelector = getRemainingSelector(node, selector2);
        addElements(node.shadowRoot, newSelector);
        findElements(node.shadowRoot, newSelector, !queriedRoot);
      }
      if (node.assignedNodes) {
        const newSelector = getRemainingSelector(node, selector2);
        const nodes = node.assignedNodes();
        for (let i = 0; i < nodes.length; i++) {
          findElements(nodes[i], newSelector, queriedRoot);
        }
      }
      if (node.childNodes && node.childNodes.length) {
        const childCount = node.childNodes.length;
        for (let i = 0; i < childCount; i++) {
          findElements(node.childNodes[i], selector2, queriedRoot);
        }
      }
    };
    findElements(root, selector);
    return Array.from(elements);
  }
  each(callback) {
    for (let index = 0; index < this.length; index++) {
      const el = this[index];
      const $el = this.chain(el);
      callback.call($el, el, index);
    }
    return this;
  }
  removeAllEvents() {
    _Query.eventHandlers = [];
  }
  find(selector) {
    const elements = Array.from(this).flatMap((el) => {
      if (this.options.pierceShadow) {
        return this.querySelectorAllDeep(el, selector, false);
      } else {
        return Array.from(el.querySelectorAll(selector));
      }
    });
    return this.chain(elements);
  }
  parent(selector) {
    const parents = Array.from(this).map((el) => el.parentElement).filter(Boolean);
    return selector ? this.chain(parents).filter(selector) : this.chain(parents);
  }
  children(selector) {
    const allChildren = Array.from(this).flatMap((el) => Array.from(el.children));
    const filteredChildren = selector ? allChildren.filter((child) => child.matches(selector)) : allChildren;
    return this.chain(filteredChildren);
  }
  siblings(selector) {
    const siblings = Array.from(this).flatMap((el) => {
      if (!el.parentNode) {
        return;
      }
      return Array.from(el.parentNode.children).filter((child) => child !== el);
    }).filter(Boolean);
    return selector ? this.chain(siblings).filter(selector) : this.chain(siblings);
  }
  // returns the index of element only including siblings that match a filter
  index(siblingFilter) {
    const el = this.el();
    if (!(el == null ? void 0 : el.parentNode)) {
      return -1;
    }
    const $siblings = this.chain(el.parentNode.children).filter(siblingFilter);
    const siblingEls = $siblings.get();
    const els = this.get();
    return findIndex(siblingEls, (el2) => inArray(el2, els));
  }
  // returns the index of current collection that match filter
  indexOf(filter) {
    const els = this.get();
    const el = this.filter(filter).get(0);
    return els.indexOf(el);
  }
  filter(filter) {
    if (!filter) {
      return this;
    }
    let filteredElements = [];
    if (isFunction(filter)) {
      filteredElements = Array.from(this).filter(filter);
    } else {
      filteredElements = Array.from(this).filter((el) => {
        if (isString(filter)) {
          return el.matches && el.matches(filter);
        } else if (filter instanceof _Query) {
          return filter.get().includes(el);
        } else {
          let els = isArray(filter) ? filter : [filter];
          return inArray(el, els);
        }
      });
    }
    return this.chain(filteredElements);
  }
  is(selector) {
    const filteredElements = Array.from(this).filter((el) => {
      if (typeof selector === "string") {
        return el.matches && el.matches(selector);
      } else if (this.isGlobal) {
        return inArray(selector, ["window", "globalThis"]);
      } else {
        const elements = selector instanceof _Query ? selector.get() : [selector];
        return elements.includes(el);
      }
    });
    return filteredElements.length === this.length;
  }
  not(selector) {
    const filteredElements = Array.from(this).filter((el) => {
      if (typeof selector === "string") {
        return !el.matches || el.matches && !el.matches(selector);
      } else if (this.isGlobal) {
        return !inArray(selector, ["window", "globalThis"]);
      } else {
        const elements = selector instanceof _Query ? selector.get() : [selector];
        return !elements.includes(el);
      }
    });
    return this.chain(filteredElements);
  }
  closest(selector) {
    const closest = Array.from(this).map((el) => {
      if (this.options.pierceShadow) {
        return this.closestDeep(el, selector);
      } else if (selector && (el == null ? void 0 : el.closest)) {
        return el.closest(selector);
      } else if (this.isGlobal) {
        return inArray(selector, ["window", "globalThis"]);
      }
    }).filter(Boolean);
    return this.chain(closest);
  }
  closestDeep(element, selector) {
    let currentElement = element;
    const domSelector = isDOM(selector);
    const stringSelector = isString(selector);
    while (currentElement) {
      if (domSelector && currentElement === selector || stringSelector && currentElement.matches(selector)) {
        return currentElement;
      }
      if (currentElement.parentElement) {
        currentElement = currentElement.parentElement;
      } else if (currentElement.parentNode && currentElement.parentNode.host) {
        currentElement = currentElement.parentNode.host;
      } else {
        return;
      }
    }
    return;
  }
  ready(handler) {
    if (this.is(document) && document.readyState == "loading") {
      this.on("ready", handler);
    } else {
      handler.call(document, new Event("DOMContentLoaded"));
    }
    return this;
  }
  getEventAlias(eventName) {
    const aliases = {
      ready: "DOMContentLoaded"
    };
    return aliases[eventName] || eventName;
  }
  getEventArray(eventNames) {
    return eventNames.split(" ").map((name) => this.getEventAlias(name)).filter(Boolean);
  }
  on(eventNames, targetSelectorOrHandler, handlerOrOptions, options) {
    const eventHandlers = [];
    let handler;
    let targetSelector;
    if (isObject(handlerOrOptions)) {
      options = handlerOrOptions;
      handler = targetSelectorOrHandler;
    } else if (isString(targetSelectorOrHandler)) {
      targetSelector = targetSelectorOrHandler;
      handler = handlerOrOptions;
    } else if (isFunction(targetSelectorOrHandler)) {
      handler = targetSelectorOrHandler;
    }
    const events = this.getEventArray(eventNames);
    events.forEach((eventName) => {
      const abortController = (options == null ? void 0 : options.abortController) || new AbortController();
      const eventSettings = (options == null ? void 0 : options.eventSettings) || {};
      const signal = abortController.signal;
      this.each((el) => {
        let delegateHandler;
        if (targetSelector) {
          delegateHandler = (event) => {
            let target;
            if (event.composed && event.composedPath) {
              let path = event.composedPath();
              const elIndex = findIndex(path, (thisEl) => thisEl == el);
              path = path.slice(0, elIndex);
              target = path.find((el2) => el2 instanceof Element && el2.matches && el2.matches(targetSelector));
            } else {
              target = event.target.closest(targetSelector);
            }
            if (target) {
              handler.call(target, event);
            }
          };
        }
        const eventListener = delegateHandler || handler;
        const domEL = el == _Query.globalThisProxy ? globalThis : el;
        if (domEL.addEventListener) {
          domEL.addEventListener(eventName, eventListener, { signal, ...eventSettings });
        }
        const eventHandler = {
          el,
          eventName,
          eventListener,
          abortController,
          delegated: targetSelector !== void 0,
          handler,
          abort: (reason) => abortController.abort(reason)
        };
        eventHandlers.push(eventHandler);
      });
    });
    if (!_Query.eventHandlers) {
      _Query.eventHandlers = [];
    }
    _Query.eventHandlers.push(...eventHandlers);
    if (options == null ? void 0 : options.returnHandler) {
      return eventHandlers.length == 1 ? eventHandlers[0] : eventHandlers;
    }
    return this;
  }
  one(eventName, targetSelectorOrHandler, handlerOrOptions, options) {
    let handler;
    let targetSelector;
    if (isObject(handlerOrOptions)) {
      options = handlerOrOptions;
      handler = targetSelectorOrHandler;
    } else if (isString(targetSelectorOrHandler)) {
      targetSelector = targetSelectorOrHandler;
      handler = handlerOrOptions;
    } else if (isFunction(targetSelectorOrHandler)) {
      handler = targetSelectorOrHandler;
    }
    options = options || {};
    const abortController = new AbortController();
    options.abortController = abortController;
    const wrappedHandler = function(...args) {
      abortController.abort();
      handler.apply(this, args);
    };
    return targetSelector ? this.on(eventName, targetSelector, wrappedHandler, options) : this.on(eventName, wrappedHandler, options);
  }
  off(eventNames, handler) {
    const events = this.getEventArray(eventNames);
    _Query.eventHandlers = _Query.eventHandlers.filter((eventHandler) => {
      if ((!eventNames || inArray(eventHandler.eventName, events)) && (!handler || (handler == null ? void 0 : handler.eventListener) == eventHandler.eventListener || eventHandler.eventListener === handler || eventHandler.handler === handler)) {
        const el = this.isGlobal ? globalThis : eventHandler.el;
        if (el.removeEventListener) {
          el.removeEventListener(eventHandler.eventName, eventHandler.eventListener);
        }
        return false;
      }
      return true;
    });
    return this;
  }
  trigger(eventName, eventParams) {
    return this.each((el) => {
      if (typeof el.dispatchEvent !== "function") {
        return;
      }
      const event = new Event(eventName, { bubbles: true, cancelable: true });
      if (eventParams) {
        Object.assign(event, eventParams);
      }
      el.dispatchEvent(event);
    });
  }
  // shorthand for most common trigger() uses
  click(eventParams) {
    return this.trigger("click", eventParams);
  }
  submit(eventParams) {
    return this.trigger("submit", eventParams);
  }
  dispatchEvent(eventName, eventData = {}, eventSettings = {}) {
    const eventOptions = {
      bubbles: true,
      cancelable: true,
      composed: true,
      detail: eventData,
      ...eventSettings
    };
    this.each((el) => {
      const event = new CustomEvent(eventName, eventOptions);
      el.dispatchEvent(event);
    });
    return this;
  }
  remove() {
    return this.each((el) => el.remove());
  }
  addClass(classNames) {
    const classesToAdd = classNames.split(" ");
    return this.each((el) => el.classList.add(...classesToAdd));
  }
  hasClass(className) {
    return Array.from(this).some((el) => el.classList.contains(className));
  }
  removeClass(classNames) {
    const classesToRemove = classNames.split(" ");
    return this.each((el) => el.classList.remove(...classesToRemove));
  }
  toggleClass(classNames) {
    const classesToToggle = classNames.split(" ");
    return this.each((el) => el.classList.toggle(...classesToToggle));
  }
  html(newHTML) {
    if (newHTML !== void 0) {
      return this.each((el) => el.innerHTML = newHTML);
    } else if (this.length > 0) {
      return this.map((el) => el.innerHTML || el.nodeValue).join("");
    }
  }
  outerHTML(newHTML) {
    if (newHTML !== void 0) {
      return this.each((el) => el.outerHTML = newHTML);
    } else if (this.length) {
      return this.map((el) => el.outerHTML).join("");
    }
  }
  getSlot(name) {
    return this.map((el) => {
      if (el.tagName.toLowerCase() == "slot" && (!name || el.name == name)) {
        const nodes = el.assignedNodes({ flatten: true });
        if (nodes) {
          return this.chain(nodes).html();
        }
      } else if (el.shadowRoot) {
        const slotSelector = name ? `slot[name="${name}"]` : "slot:not([name])";
        const slot = el.shadowRoot.querySelector(slotSelector);
        const nodes = slot.assignedNodes({ flatten: true });
        if (nodes) {
          return this.chain(nodes).html();
        }
      } else {
        const slotSelector = name ? `[slot="${name}"]` : ":not([slot])";
        return this.chain(el).find(slotSelector).html();
      }
    }).join("");
  }
  setSlot(nameOrHTML, newHTML) {
    let name;
    if (newHTML) {
      name = nameOrHTML;
    } else {
      newHTML = nameOrHTML;
    }
    return this.each((el) => {
      var _a;
      if (el.tagName.toLowerCase() == "slot") {
        el = (_a = el.getRootNode().getRootNode()) == null ? void 0 : _a.host;
      }
      const $el = this.chain(el);
      if (name) {
        const slotSelector = `[slot="${name}"]`;
        let $slottedElement = this.chain(el).find(slotSelector);
        if (!$slottedElement.exists()) {
          $el.append(`<span slot="${name}"></span>`);
          $slottedElement = this.chain(el).find(slotSelector);
        }
        $slottedElement.html(newHTML);
      } else {
        $el.html(newHTML);
      }
    });
  }
  text(newText) {
    if (newText !== void 0) {
      return this.each((el) => el.textContent = newText);
    } else {
      const childNodes = (el) => {
        return el.nodeName === "SLOT" ? el.assignedNodes({ flatten: true }) : el.childNodes;
      };
      const values = this.map((el) => this.getTextContentRecursive(childNodes(el)));
      return values.length > 1 ? values : values[0];
    }
  }
  // Helper function to recursively get text content
  getTextContentRecursive(nodes) {
    return Array.from(nodes).map((node) => {
      if (node.nodeType === Node.TEXT_NODE) {
        return node.nodeValue;
      } else if (node.nodeName === "SLOT") {
        const slotNodes = node.assignedNodes({ flatten: true });
        return this.getTextContentRecursive(slotNodes);
      } else {
        return this.getTextContentRecursive(node.childNodes);
      }
    }).join("").trim();
  }
  // non jquery variant to return only immediate text node
  textNode() {
    return Array.from(this).map((el) => {
      return Array.from(el.childNodes).filter((node) => node.nodeType === Node.TEXT_NODE).map((node) => node.nodeValue).join("");
    }).join("");
  }
  map(...args) {
    return Array.from(this).map(...args);
  }
  value(newValue) {
    const usesValue = (el) => {
      return el instanceof HTMLInputElement || el instanceof HTMLSelectElement || el instanceof HTMLTextAreaElement || customElements.get(el.tagName.toLowerCase());
    };
    if (newValue !== void 0) {
      return this.each((el) => {
        if (usesValue(el)) {
          el.value = newValue;
        }
      });
    } else {
      const values = this.map((el) => {
        if (usesValue(el)) {
          return el.value;
        }
        return void 0;
      });
      return values.length > 1 ? values : values[0];
    }
  }
  // alias
  val(...args) {
    return this.value(...args);
  }
  focus() {
    if (this.length) {
      this[0].focus();
    }
    return this;
  }
  blur() {
    if (this.length) {
      this[0].blur();
    }
    return this;
  }
  css(property, value = null, settings = { includeComputed: false }) {
    const elements = Array.from(this);
    if (isPlainObject(property) || value !== null) {
      if (isPlainObject(property)) {
        Object.entries(property).forEach(([prop, val]) => {
          elements.forEach((el) => el.style.setProperty(camelToKebab(prop), val));
        });
      } else {
        elements.forEach((el) => el.style.setProperty(camelToKebab(property), value));
      }
      return this;
    } else {
      if (elements == null ? void 0 : elements.length) {
        const styles = elements.map((el) => {
          const inlineStyle = el.style[property];
          if (settings.includeComputed) {
            return window.getComputedStyle(el).getPropertyValue(property);
          }
          if (inlineStyle) {
            return inlineStyle;
          }
          return void 0;
        });
        return elements.length === 1 ? styles[0] : styles;
      }
    }
  }
  computedStyle(property) {
    return this.css(property, null, { includeComputed: true });
  }
  cssVar(variable, value) {
    return this.css(`--${variable}`, value, { includeComputed: true });
  }
  attr(attribute, value) {
    if (isPlainObject(attribute)) {
      Object.entries(attribute).forEach(([attr, val]) => {
        this.each((el) => el.setAttribute(attr, val));
      });
    } else if (value !== void 0) {
      this.each((el) => el.setAttribute(attribute, value));
    } else if (this.length) {
      const attributes = this.map((el) => el.getAttribute(attribute));
      return attributes.length > 1 ? attributes : attributes[0];
    }
    return;
  }
  removeAttr(attributeName) {
    return this.each((el) => el.removeAttribute(attributeName));
  }
  el() {
    return this.get(0);
  }
  get(index) {
    if (index !== void 0) {
      return this[index];
    } else {
      return Array.from(this);
    }
  }
  eq(index) {
    return this.chain(this[index]);
  }
  first() {
    return this.eq(0);
  }
  last() {
    return this.eq(this.length - 1);
  }
  prop(name, value) {
    if (value !== void 0) {
      return this.each((el) => {
        el[name] = value;
      });
    } else {
      if (this.length == 0) {
        return void 0;
      } else if (this.length === 1) {
        return this[0][name];
      } else {
        return this.map((el) => el[name]);
      }
    }
  }
  next(selector) {
    const nextSiblings = this.map((el) => {
      let nextSibling = el.nextElementSibling;
      while (nextSibling) {
        if (!selector || nextSibling.matches(selector)) {
          return nextSibling;
        }
        nextSibling = nextSibling.nextElementSibling;
      }
      return null;
    }).filter(Boolean);
    return this.chain(nextSiblings);
  }
  prev(selector) {
    const prevSiblings = this.map((el) => {
      let prevSibling = el.previousElementSibling;
      while (prevSibling) {
        if (!selector || prevSibling.matches(selector)) {
          return prevSibling;
        }
        prevSibling = prevSibling.previousElementSibling;
      }
      return null;
    }).filter(Boolean);
    return this.chain(prevSiblings);
  }
  height(value) {
    return this.prop("innerHeight", value) || this.prop("clientHeight", value);
  }
  width(value) {
    return this.prop("innerWidth", value) || this.prop("clientWidth", value);
  }
  scrollHeight(value) {
    const el = this.isGlobal && this.isBrowser ? this.chain(document.documentElement) : this;
    return el.prop("scrollHeight", value);
  }
  scrollWidth(value) {
    const el = this.isGlobal && this.isBrowser ? this.chain(document.documentElement) : this;
    return el.prop("scrollWidth", value);
  }
  scrollLeft(value) {
    const el = this.isGlobal && this.isBrowser ? this.chain(document.documentElement) : this;
    return el.prop("scrollLeft", value);
  }
  scrollTop(value) {
    const el = this.isGlobal && this.isBrowser ? this.chain(document.documentElement) : this;
    return el.prop("scrollTop", value);
  }
  clone() {
    const fragment = document.createDocumentFragment();
    this.each((el) => {
      fragment.appendChild(el.cloneNode(true));
    });
    return this.chain(fragment.childNodes);
  }
  reverse() {
    const els = this.get().reverse();
    return this.chain(els);
  }
  insertContent(target, content, position) {
    const $content = this.chain(content);
    $content.each((el) => {
      var _a, _b;
      if (target.insertAdjacentElement) {
        target.insertAdjacentElement(position, el);
      } else {
        switch (position) {
          case "beforebegin":
            (_a = target.parentNode) == null ? void 0 : _a.insertBefore(el, target);
            break;
          case "afterbegin":
            target.insertBefore(el, target.firstChild);
            break;
          case "beforeend":
            target.appendChild(el);
            break;
          case "afterend":
            (_b = target.parentNode) == null ? void 0 : _b.insertBefore(el, target.nextSibling);
            break;
        }
      }
    });
  }
  prepend(...allContent) {
    return this.each((el) => {
      each(allContent, (content) => {
        this.insertContent(el, content, "afterbegin");
      });
    });
  }
  append(...allContent) {
    return this.each((el) => {
      each(allContent, (content) => {
        this.insertContent(el, content, "beforeend");
      });
    });
  }
  insertBefore(selector) {
    return this.chain(selector).each((el) => {
      this.insertContent(el, this.selector, "beforebegin");
    });
  }
  insertAfter(selector) {
    return this.chain(selector).each((el) => {
      this.insertContent(el, this.selector, "afterend");
    });
  }
  detach() {
    return this.each((el) => {
      if (el.parentNode) {
        el.parentNode.removeChild(el);
      }
    });
  }
  naturalWidth() {
    const widths = this.map((el) => {
      const $clone = $(el).clone();
      $clone.insertAfter(el).css({
        position: "absolute",
        display: "block",
        transform: "translate(-9999px, -9999px)",
        zIndex: "-1"
      });
      const naturalWidth = $clone.width();
      $clone.remove();
      return naturalWidth;
    });
    return widths.length > 1 ? widths : widths[0];
  }
  naturalHeight() {
    const height = this.map((el) => {
      const $clone = $(el).clone();
      $clone.insertAfter(el).css({
        position: "absolute",
        display: "block",
        transform: "translate(-9999px, -9999px)",
        zIndex: "-1"
      });
      const naturalHeight = $clone.height();
      $clone.remove();
      return naturalHeight;
    });
    return height.length > 1 ? height : height[0];
  }
  // offsetParent does not return the true offset parent
  // in cases where there is a parent node with a transform context
  // so we need to get that manually where finding the true offset parent is essential
  // for instance when calculating position
  offsetParent({ calculate = true } = {}) {
    return Array.from(this).map((el) => {
      if (!calculate) {
        return el.offsetParent;
      }
      let $el, isPositioned, isTransformed, isBody;
      let parentNode = el == null ? void 0 : el.parentNode;
      while (parentNode && !isPositioned && !isTransformed && !isBody) {
        parentNode = parentNode == null ? void 0 : parentNode.parentNode;
        if (parentNode) {
          $el = $(parentNode);
          isPositioned = $el.computedStyle("position") !== "static";
          isTransformed = $el.computedStyle("transform") !== "none";
          isBody = $el.is("body");
        }
      }
      return parentNode;
    });
  }
  // alias
  count() {
    return this.length;
  }
  exists() {
    return this.length > 0;
  }
  // adds properties to an element after dom loads
  initialize(settings) {
    document.addEventListener("DOMContentLoaded", () => {
      this.settings(settings);
    });
  }
  settings(settings) {
    return this.each((el) => {
      each(settings, (value, setting) => {
        el[setting] = value;
      });
    });
  }
  setting(setting, value) {
    return this.each((el) => {
      el[setting] = value;
    });
  }
  // special helper for SUI components
  component() {
    const components = this.map((el) => el.component).filter(Boolean);
    return components.length > 1 ? components : components[0];
  }
  dataContext() {
    const contexts = this.map((el) => el.dataContext).filter(Boolean);
    return contexts.length > 1 ? contexts : contexts[0];
  }
};
/*
  This avoids keeping a copy of window/globalThis in
  memory when an element references the global object
  reducing memory footprint
*/
__publicField(_Query, "globalThisProxy", new Proxy({}, {
  get(target, prop) {
    return globalThis[prop];
  },
  set(target, prop, value) {
    globalThis[prop] = value;
    return true;
  }
}));
/*
  We keep an array of event handlers for teardown
*/
__publicField(_Query, "eventHandlers", []);
var Query = _Query;

// src/index.js
var $2 = function(selector, args = {}) {
  const isClient2 = typeof window !== "undefined";
  if (!(args == null ? void 0 : args.root) && isClient2) {
    args.root = document;
  }
  return new Query(selector, args);
};
var $$ = function(selector, args = {}) {
  args.pierceShadow = true;
  return $2(selector, args);
};
var originalDollar;
var originalDoubleDollar;
var exportGlobals = function({ dollar = true, doubleDollar = true, query = true } = {}) {
  if (isClient) {
    if (dollar) {
      originalDollar = window.$;
      window.$ = $2;
    }
    if (doubleDollar) {
      originalDoubleDollar = window.$$;
      window.$$ = $$;
    }
    if (query) {
      window.Query = Query;
    }
  }
};
var restoreGlobals = function(settings) {
  if (window.$ === $2) {
    window.$ = originalDollar;
  }
  if (window.$$ === $$) {
    window.$$ = originalDoubleDollar;
  }
  if (typeof settings == "object" && settings.removeQuery && window.Query === Query) {
    window.Query = void 0;
  }
  return $2;
};
var useAlias = function() {
  return new Query(...arguments);
};
export {
  $2 as $,
  $$,
  Query,
  exportGlobals,
  restoreGlobals,
  useAlias
};
//# sourceMappingURL=query.js.map
